//+------------------------------------------------------------------+
//|                                           Schaff Trend Cycle.mq4 |
//|                                                           mladen |
//+------------------------------------------------------------------+
#property copyright "mladen"
#property link      "mladenfx@gmail.com"

#property indicator_separate_window
#property indicator_buffers    1
#property indicator_color1     Orange
#property indicator_width1     2
#property indicator_minimum    -5
#property indicator_maximum    105
#property indicator_levelcolor MediumOrchid


//
//
//
//
//

extern string TimeFrame       = "current time frame";
extern int    STCPeriod       = 15;
extern int    TrixPeriod      = 30;
extern int    Price           = PRICE_MEDIAN; 

extern double levelOb         = 80;
extern double levelOs         = 20;

extern bool   alertsOn        = false;
extern bool   alertsOnSlope   = true;
extern bool   alertsOnOsOb    = true;
extern bool   alertsOnCurrent = true;
extern bool   alertsMessage   = true;
extern bool   alertsSound     = false;
extern bool   alertsNotify    = false;
extern bool   alertsEmail     = false;

//
//
//
//
//

double stcBuffer[];
double trixBuffer[];
double fastKBuffer[];
double fastDBuffer[];
double fastKKBuffer[];
double ema3[];
double trend[];
double slope[];

//
//
//
//
//

string indicatorFileName;
bool   calculateValue;
bool   returnBars;
int    timeFrame;

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int init()
{
   IndicatorBuffers(8);
   SetIndexBuffer(0,stcBuffer);
   SetIndexBuffer(1,trixBuffer);
   SetIndexBuffer(2,fastKBuffer);
   SetIndexBuffer(3,fastDBuffer);
   SetIndexBuffer(4,fastKKBuffer);
   SetIndexBuffer(5,ema3);
   SetIndexBuffer(6,slope);
   SetIndexBuffer(7,trend);
   SetLevelValue (0,levelOb);     
   SetLevelValue (1,levelOs);       
   
      //
      //
      //
      //
      //
   
      indicatorFileName = WindowExpertName();
      returnBars        = TimeFrame=="returnBars";     if (returnBars)     return(0);
      calculateValue    = TimeFrame=="calculateValue"; if (calculateValue) return(0);
      timeFrame         = stringToTimeFrame(TimeFrame);
      
  IndicatorShortName(timeFrameToString(timeFrame)+"   Schaff Trend Cycle ("+STCPeriod+","+TrixPeriod+")");  
return(0);
}

int deinit()
{
   return(0);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int start()
{
   int counted_bars=IndicatorCounted();
   int limit,i;

   if(counted_bars < 0) return(-1);
   if(counted_bars > 0) counted_bars--;
           limit=MathMin(Bars-1,Bars-counted_bars-1);
           if (returnBars)  { stcBuffer[0] = limit+1; return(0); }
           
   //
   //
   //
   //
   //
    
   if (calculateValue || timeFrame==Period())
   {      
      for(i = limit; i >= 0; i--)
      {
      double price = MathLog(iMA(NULL,0,1,0,MODE_SMA,Price,i));
         double ema1    = iEma(price,TrixPeriod,i,0);
         double ema2    = iEma(ema1 ,TrixPeriod,i,1);
                ema3[i] = iEma(ema2 ,TrixPeriod,i,2);
         trixBuffer[i]  = (ema3[i]-ema3[i+1])*10000;

         //
         //
         //
         //
         //
         
         double lowtrix  = minValue(trixBuffer,i);
         double hightrix = maxValue(trixBuffer,i)-lowtrix;
            if (hightrix > 0)
                  fastKBuffer[i] = 100*((trixBuffer[i]-lowtrix)/hightrix);
            else  fastKBuffer[i] = fastKBuffer[i+1];
                  fastDBuffer[i] = fastDBuffer[i+1]+0.5*(fastKBuffer[i]-fastDBuffer[i+1]);
               
         //
         //
         //
         //
         //
                     
         double lowStoch  = minValue(fastDBuffer,i);
         double highStoch = maxValue(fastDBuffer,i)-lowStoch;
            if (highStoch > 0)
                  fastKKBuffer[i] = 100*((fastDBuffer[i]-lowStoch)/highStoch);
            else  fastKKBuffer[i] = fastKKBuffer[i+1];
                  stcBuffer[i]    = stcBuffer[i+1]+0.5*(fastKKBuffer[i]-stcBuffer[i+1]);
                  slope[i] = slope[i+1];
                  trend[i] = trend[i+1];
                  if (stcBuffer[i] > stcBuffer[i+1]) slope[i] = 1;
                  if (stcBuffer[i] < stcBuffer[i+1]) slope[i] =-1;
                  if (stcBuffer[i] < levelOs)        trend[i] = 1;
                  if (stcBuffer[i] > levelOb)        trend[i] =-1;
                  
      }   
      manageAlerts();
      return(0);
    }
    
   //
   //
   //
   //
   //
      
   limit = MathMax(limit,MathMin(Bars-1,iCustom(NULL,timeFrame,indicatorFileName,"returnBars",0,0)*timeFrame/Period()));
   for(i=limit; i>=0; i--)
   {
      int y = iBarShift(NULL,timeFrame,Time[i]);
         stcBuffer[i] = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",STCPeriod,TrixPeriod,Price,0,y);
         slope[i]     = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",STCPeriod,TrixPeriod,Price,6,y);
         trend[i]     = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",STCPeriod,TrixPeriod,Price,7,y);
   }
   manageAlerts();    
return(0);            
}

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

double workEma[][3];
double iEma(double price, double period, int r, int instanceNo=0)
{
   if (ArrayRange(workEma,0)!= Bars) ArrayResize(workEma,Bars); r = Bars-r-1;

   //
   //
   //
   //
   //
      
   double alpha = 2.0 / (1.0+period);
          workEma[r][instanceNo] = workEma[r-1][instanceNo]+alpha*(price-workEma[r-1][instanceNo]);
   return(workEma[r][instanceNo]);
} 

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

double minValue(double& array[],int shift)
{
   double minValue = array[shift];
            for (int i=1; i<STCPeriod; i++) minValue = MathMin(minValue,array[shift+i]);
   return(minValue);
}
double maxValue(double& array[],int shift)
{
   double maxValue = array[shift];
            for (int i=1; i<STCPeriod; i++) maxValue = MathMax(maxValue,array[shift+i]);
   return(maxValue);
}

//+-------------------------------------------------------------------
//|                                                                  
//+-------------------------------------------------------------------
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

//
//
//
//
//

int stringToTimeFrame(string tfs)
{
   tfs = StringUpperCase(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
         if (tfs==sTfTable[i] || tfs==""+iTfTable[i]) return(MathMax(iTfTable[i],Period()));
                                                      return(Period());
}

//
//
//
//
//

string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}

//
//
//
//
//

string StringUpperCase(string str)
{
   string   s = str;

   for (int length=StringLen(str)-1; length>=0; length--)
   {
      int char = StringGetChar(s, length);
         if((char > 96 && char < 123) || (char > 223 && char < 256))
                     s = StringSetChar(s, length, char - 32);
         else if(char > -33 && char < 0)
                     s = StringSetChar(s, length, char + 224);
   }
   return(s);
}

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

void manageAlerts()
{
   if (alertsOn)
   {
      if (alertsOnCurrent)
           int whichBar = 0;
      else     whichBar = 1; whichBar = iBarShift(NULL,0,iTime(NULL,timeFrame,whichBar));
      
      //
      //
      //
      //
      //
      
      static datetime time1 = 0;
      static string   mess1 = "";
         if (alertsOnSlope && slope[whichBar] != slope[whichBar+1])
         {
            if (slope[whichBar] ==  1) doAlert(time1,mess1,whichBar," Schaff Trix Cycle sloping up");
            if (slope[whichBar] == -1) doAlert(time1,mess1,whichBar," Schaff Trix Cycle sloping down");
         }            
      static datetime time2 = 0;
      static string   mess2 = "";
         if (alertsOnOsOb && trend[whichBar] != trend[whichBar+1])
         {
            if (trend[whichBar] ==  1) doAlert(time2,mess2,whichBar," Schaff Trix Cycle entering oversold zone");
            if (trend[whichBar] == -1) doAlert(time2,mess2,whichBar," Schaff Trix Cycle entering overbought zone");
         }            
   }
}   

//
//
//
//
//

void doAlert(datetime& previousTime, string& previousAlert, int forBar, string doWhat)
{
   string message;
   
      if (previousAlert != doWhat || previousTime != Time[forBar]) {
          previousAlert  = doWhat;
          previousTime   = Time[forBar];

          //
          //
          //
          //
          //

          message =  Symbol()+" "+timeFrameToString(timeFrame)+" at "+TimeToStr(TimeLocal(),TIME_SECONDS)+doWhat;
             if (alertsMessage) Alert(message);
             if (alertsNotify)  SendNotification(message);
             if (alertsEmail)   SendMail(StringConcatenate(Symbol()," Schaff Trend Cycle "),message);
             if (alertsSound)   PlaySound("alert2.wav");
      }
}
   